@@ -50,6 +50,7 @@ from awscli.compat import six
 from botocore.hooks import HierarchicalEmitter
 from botocore.session import Session
 from botocore.exceptions import ClientError
+from botocore.exceptions import WaiterError
 import botocore.loaders
 from botocore.vendored import requests
 
@@ -463,6 +464,58 @@ class BaseAWSPreviewCommandParamsTest(BaseAWSCommandParamsTest):
         super(BaseAWSPreviewCommandParamsTest, self).tearDown()
 
 
+class BaseCLIWireResponseTest(unittest.TestCase):
+    def setUp(self):
+        self.environ = {
+            'AWS_DATA_PATH': os.environ['AWS_DATA_PATH'],
+            'AWS_DEFAULT_REGION': 'us-east-1',
+            'AWS_ACCESS_KEY_ID': 'access_key',
+            'AWS_SECRET_ACCESS_KEY': 'secret_key',
+            'AWS_CONFIG_FILE': ''
+        }
+        self.environ_patch = mock.patch('os.environ', self.environ)
+        self.environ_patch.start()
+        self.send_patch = mock.patch('botocore.endpoint.Session.send')
+        self.send_is_patched = False
+        self.driver = create_clidriver()
+
+    def tearDown(self):
+        self.environ_patch.stop()
+        if self.send_is_patched:
+            self.send_patch.stop()
+            self.send_is_patched = False
+
+    def patch_send(self, status_code=200, headers={}, content=b''):
+        if self.send_is_patched:
+            self.patch_send.stop()
+            self.send_is_patched = False
+        send_patch = self.send_patch.start()
+        send_patch.return_value = mock.Mock(status_code=status_code,
+                                            headers=headers,
+                                            content=content)
+        self.send_is_patched = True
+
+    def run_cmd(self, cmd, expected_rc=0):
+        if not isinstance(cmd, list):
+            cmdlist = cmd.split()
+        else:
+            cmdlist = cmd
+        with capture_output() as captured:
+            try:
+                rc = self.driver.main(cmdlist)
+            except SystemExit as e:
+                rc = e.code
+        stderr = captured.stderr.getvalue()
+        stdout = captured.stdout.getvalue()
+        self.assertEqual(
+            rc, expected_rc,
+            "Unexpected rc (expected: %s, actual: %s) for command: %s\n"
+            "stdout:\n%sstderr:\n%s" % (
+                expected_rc, rc, cmd, stdout, stderr))
+        return stdout, stderr, rc
+
+
+
 class FileCreator(object):
     def __init__(self):
         self.rootdir = tempfile.mkdtemp()
@@ -694,6 +747,10 @@ class BaseS3CLICommand(unittest.TestCase):
         # Subclasses can use this to define extra teardown steps.
         pass
 
+    def override_parser(self, **kwargs):
+        factory = self.session.get_component('response_parser_factory')
+        factory.set_parser_defaults(**kwargs)
+
     def create_client_for_bucket(self, bucket_name):
         region = self.regions.get(bucket_name, self.region)
         client = self.session.create_client('s3', region_name=region)
@@ -754,12 +811,20 @@ class BaseS3CLICommand(unittest.TestCase):
         response = client.get_object(Bucket=bucket_name, Key=key_name)
         return response['Body'].read().decode('utf-8')
 
-    def key_exists(self, bucket_name, key_name):
-        client = self.create_client_for_bucket(bucket_name)
+    def key_exists(self, bucket_name, key_name, min_successes=3):
         try:
-            client.head_object(Bucket=bucket_name, Key=key_name)
+            self.wait_until_key_exists(
+                    bucket_name, key_name, min_successes=min_successes)
             return True
-        except ClientError:
+        except (ClientError, WaiterError):
+            return False
+
+    def key_not_exists(self, bucket_name, key_name, min_successes=3):
+        try:
+            self.wait_until_key_not_exists(
+                    bucket_name, key_name, min_successes=min_successes)
+            return True
+        except (ClientError, WaiterError):
             return False
 
     def list_buckets(self):
@@ -777,8 +842,21 @@ class BaseS3CLICommand(unittest.TestCase):
 
     def wait_until_key_exists(self, bucket_name, key_name, extra_params=None,
                               min_successes=3):
+        self._wait_for_key(bucket_name, key_name, extra_params,
+                           min_successes, exists=True)
+
+    def wait_until_key_not_exists(self, bucket_name, key_name, extra_params=None,
+                                  min_successes=3):
+        self._wait_for_key(bucket_name, key_name, extra_params,
+                           min_successes, exists=False)
+
+    def _wait_for_key(self, bucket_name, key_name, extra_params=None,
+                      min_successes=3, exists=True):
         client = self.create_client_for_bucket(bucket_name)
-        waiter = client.get_waiter('object_exists')
+        if exists:
+            waiter = client.get_waiter('object_exists')
+        else:
+            waiter = client.get_waiter('object_not_exists')
         params = {'Bucket': bucket_name, 'Key': key_name}
         if extra_params is not None:
             params.update(extra_params)
